# Copyright ETSI Contributors and Others.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import click
from osmclient.cli_commands import common
import logging

logger = logging.getLogger("osmclient")


@click.command(name="cluster-create", short_help="creates a K8s cluster")
@click.argument("name")
@click.option("--node-count", "-n", prompt=True, type=int, help="number of nodes")
@click.option("--node-size", prompt=True, help="size of the worker nodes")
@click.option("--version", prompt=True, help="Kubernetes version")
@click.option(
    "--vim-account",
    prompt=True,
    help="VIM target or cloud account where the cluster will be created",
)
@click.option("--description", default="", help="human readable description")
@click.option(
    "--region-name",
    default=None,
    help="region name in VIM account where the cluster will be created",
)
@click.option(
    "--resource-group",
    default=None,
    help="resource group in VIM account where the cluster will be created",
)
@click.pass_context
def cluster_create(
    ctx,
    name,
    node_count,
    node_size,
    version,
    vim_account,
    description,
    region_name,
    resource_group,
    **kwargs
):
    """creates a K8s cluster

    NAME: name of the K8s cluster
    """
    logger.debug("")
    # kwargs = {k: v for k, v in kwargs.items() if v is not None}
    cluster = {}
    cluster["name"] = name
    cluster["node_count"] = node_count
    cluster["node_size"] = node_size
    cluster["k8s_version"] = version
    cluster["vim_account"] = vim_account
    if description:
        cluster["description"] = description
    if region_name:
        cluster["region_name"] = region_name
    if resource_group:
        cluster["resource_group"] = resource_group
    ctx.obj.cluster.create(name, content_dict=cluster)


@click.command(name="cluster-delete", short_help="deletes a K8s cluster")
@click.argument("name")
@click.option(
    "--force", is_flag=True, help="forces the deletion from the DB (not recommended)"
)
@click.pass_context
def cluster_delete(ctx, name, force):
    """deletes a K8s cluster

    NAME: name or ID of the K8s cluster to be deleted
    """
    logger.debug("")
    ctx.obj.cluster.delete(name, force=force)


@click.command(name="cluster-list")
@click.option(
    "--filter",
    help="restricts the list to the items matching the filter",
)
@click.pass_context
def cluster_list(ctx, filter):
    """list all K8s clusters"""
    logger.debug("")
    extras = {"Created": "created"}
    common.generic_list(callback=ctx.obj.cluster.list, filter=filter, extras=extras)


@click.command(
    name="cluster-show",
    short_help="shows the details of a K8s cluster",
)
@click.argument("name")
@click.pass_context
def cluster_show(ctx, name):
    """shows the details of a K8s cluster

    NAME: name or ID of the K8s cluster
    """
    logger.debug("")
    common.generic_show(callback=ctx.obj.cluster.get, name=name)


@click.command(name="cluster-update", short_help="updates a K8s cluster")
@click.argument("name")
@click.option("--newname", help="New name for the K8s cluster")
@click.option("--description", help="human readable description")
@click.pass_context
def cluster_update(ctx, name, newname, description, **kwargs):
    """updates a K8s cluster

    NAME: name or ID of the K8s cluster
    """
    logger.debug("")
    cluster_changes = common.generic_update(newname, description, kwargs)
    ctx.obj.cluster.update(name, changes_dict=cluster_changes)


@click.command(
    name="cluster-get-credentials", short_help="get kubeconfig of a K8s cluster"
)
@click.argument("name")
@click.pass_context
def cluster_get_credentials(ctx, name, **kwargs):
    """updates a K8s cluster

    NAME: name or ID of the K8s cluster
    """
    logger.debug("")
    ctx.obj.cluster.get_credentials(name)
